// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancymysql.h 2 2006-01-14 14:09:11Z wglas $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** A persistent mysql connection with a controlling thread.
** 
*****************************************************************************/

#ifndef FANCYMYSQL_H
#define FANCYMYSQL_H

#include <string>
#include <fancythread.h>
#include <fancytimeval.h>
#include <mysql/mysql.h>

namespace Fancy {
  namespace MySQL {

    class ConnectionPoolRec;

    class Connection : public Fancy::Thread
    {
      friend class Transaction;
      friend class Result;

    public:
      Connection(int pool_sz=1);

      ~Connection();

      void connect (const char *host,
                    const char *user,
                    const char *passwd,
                    const char *db,
                    int _ping_interval=0,
                    int _server_timeout=0);

      std::string errorMessage();

      static Connection *defaultConnection();

    protected:
      void run();

    private:
      MYSQL *aquireHandle(int &id);
      void releaseHandle(int id);

      int ping_interval;
      int server_timeout;
      bool connected;
      ConnectionPoolRec *pool;
      int pool_size;
      int next_pool_id;
      Fancy::Mutex mutex;
      struct timeval last_success;

      static Connection *default_connection;
    };

    class Transaction
    {
      friend class Result;
    public:
      Transaction (bool read_only=false, Connection *conn=0);

      ~Transaction ();

      std::string quote(const std::string &s);

      std::string nextId();

      void rawQuery(const std::string &query);
      void query(const std::string &query, const std::string &person_id);
      void commit();

    private:
      Connection *connection;
      bool ro;
      MYSQL *mysql;
      int id;
    };

    class Result
    {
    public:
      Result (Transaction &txn);

      ~Result ();

      MYSQL_RES *handle();
      MYSQL_ROW fetchRow();
      unsigned long *fetchLengths();
      int numFields();

      std::string value (int i);

    private:
      MYSQL_RES *res;
      MYSQL_ROW row;
    };
  }
}

#endif
